/*
 * Breaking the Multi Colored Box - Program Entry Point (Main)
 * 
 * Sumit Khanna 
 * http://penguindreams.org
 * 
 * Created on Aug 3, 2007
 */
package org.penguindreams.bmcb;

import java.io.IOException;

import javax.swing.JFrame;

import org.apache.log4j.Logger;
import org.penguindreams.bmcb.gui.SegmentViewer;
import org.penguindreams.bmcb.util.ApplicationLogger;
import org.penguindreams.bmcb.util.ConfigMap;
import org.penguindreams.bmcb.workflows.DatasetGeneratorWorkflow;
import org.penguindreams.bmcb.workflows.ImageMagickFilterTestWorkflow;
import org.penguindreams.bmcb.workflows.TrivialTestWorkflow;

/**
 * Program Entry Point.
 * @author Sumit Khanna
 */
public class BMCBMain {
	
	
	public static ConfigMap config;

	static void printUsage() {
		String s = "Usage: Bmcb [generate|trivialTest|magickTest|SegmentDebug]\n";
		System.err.println(s);
	}
	
	public static void main(String[] args) {
		
		ApplicationLogger.setLoggerConfig("log4j.properities");
		Logger log = ApplicationLogger.getLogger(BMCBMain.class);
		log.info("Starting Breaking the Multi Colored Box");
		
		//load config
		try {
			config = new ConfigMap("bmcb.config");
		} catch (IOException e) {
			log.fatal("Could Not Load Config File " + e.getMessage());
			System.exit(2);
		}
		
		if(args.length == 0) {
			printUsage();
			System.exit(1);
		}
		else if(args[0].equals("generate")) {
			DatasetGeneratorWorkflow wf = new DatasetGeneratorWorkflow();
			wf.execute();
		}
		else if(args[0].equals("trivialTest")) {
			TrivialTestWorkflow wf = new TrivialTestWorkflow();
			wf.execute();
		}
		else if(args[0].equals("magickTest")) {
			ImageMagickFilterTestWorkflow wf = new ImageMagickFilterTestWorkflow();
			wf.execute();
		}
		else if(args[0].equals("SegmentDebug")) {
			
			JFrame jf_main = new JFrame("Segmentator Tester");
			jf_main.setSize(800,480);
			jf_main.getContentPane().add(new SegmentViewer());
			jf_main.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
			jf_main.setVisible(true);
		}
		
		
	}
	

}
