package org.penguindreams.bmcb.analyzers;

import java.awt.Image;
import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;

import net.sourceforge.jiu.ops.MissingParameterException;
import net.sourceforge.jiu.ops.OperationFailedException;

import org.penguindreams.bmcb.BMCBMain;
import org.penguindreams.bmcb.util.ImageLoader;
import org.penguindreams.bmcb.util.InvalidConfigException;

public abstract class AbstractCommandLineAnalyzer extends AbstractAnalyzer {

	private String cmd;
	
	private String name;
	
	public AbstractCommandLineAnalyzer(String name, String configValue) throws AnalyzerNotFoundException {
		super();

		this.name = name;
		
		try {
			cmd = BMCBMain.config.getConfigValue(configValue);
		} catch (InvalidConfigException e) {
			throw new AnalyzerNotFoundException("The " + name + " Tool could not be found. "+
			  "Please make sure " + configValue + " is defined in bmcb.config with a "+ 
			  "full path to its executable");
		}

		
	}
	
	public String getName() {
		return name;
	}
	
	protected String[] runExternalAnalyzer(String[] args) {
		ArrayList<String> retval = new ArrayList<String>();
		
		long starttime = System.currentTimeMillis();
		log.debug("Running Externanal Analyzer " + name + " using command " + cmd);

		String[] cmdLine = new String[args.length+1]; //gotta add the actual command in arg[0]
		cmdLine[0] = cmd;
		for(int i=0; i<args.length; i++) {
			log.trace("Argument #" +i + " is "  + args[i]);
			cmdLine[i+1] = args[i]; //add the other commands
		}
		
		try {
			Process p = Runtime.getRuntime().exec(cmdLine);
			BufferedReader stdout = new BufferedReader(new InputStreamReader(p.getInputStream()));
			
			String s = null;
			while( (s = stdout.readLine()) != null) {
				retval.add(s);
			}
			
			p.waitFor();
		} catch (IOException e) {
			log.error("I/O Error When Executing " + cmd);
			e.printStackTrace();
		} catch (InterruptedException e) {
			e.printStackTrace();
			log.error("Process Interrupted When Executing " + cmd);
		}
		
		
		long total = System.currentTimeMillis() - starttime;
		log.debug("Analysis completed in " + total +"ms");
		return retval.toArray(new String[0]);
	}
	
	/**
	 * simple analyzer function that is useful for most OCR software.
	 * Creates a temporary PNM file based on the input Image and calls
	 * the external command on the PNM file, parses the output and returns it in a
	 * character array excluding any spaces. Useful for most common command line OCR
	 * utilities. 
	 * @param inputImage
	 * @return
	 */
	protected char[] straightAnalysis(Image inputImage) {
		File f = null;
		try {
			f = File.createTempFile("BMCB", ".PNM");
			ImageLoader.saveToPNM(inputImage,f);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (MissingParameterException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (OperationFailedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		String[] r = {f.getAbsolutePath()};
		String[] ret = runExternalAnalyzer(r);
		
		String retstring = "";
		for(String i : ret) {
			for(char j : i.trim().toCharArray()) {
				if(j != '\0' && j != '\n' && j != ' ' ) {
					retstring += j;
				}
 			}
		}
		if(!f.delete()) {
			log.warn("Could not delete tmp file " + f);
		}
		return retstring.toCharArray(); 
	}
	
	/*
	@Override
	public char[] analyzeImage(Image inputImage)
			throws AnalysisNotSupportedException {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public char[] analyzeSegments(Image[] segments)
			throws AnalysisNotSupportedException {
		// TODO Auto-generated method stub
		return null;
	}
	*/

	@Override
	public void trainUsingImage(Image img, char[] answer)
			throws AnalysisNotSupportedException {
		throw new AnalysisNotSupportedException(name + " Analyzer Does Not Support Training");

	}

	@Override
	public void trainUsingSegments(Image[] segments, char[] answer)
			throws AnalysisNotSupportedException {
		throw new AnalysisNotSupportedException(name + " Analyzer Does Not Support Training");

	}

}
