/*
 * Breaking the Multi Colored Box - DataSet Class
 * 
 * Sumit Khanna 
 * http://penguindreams.org
 * 
 * Created on Aug 3, 2007
 *
 */
package org.penguindreams.bmcb.data;

import java.util.Random;

/**
 *  A class designed to generate and handle datasets for BMCB.
 * @author Sumit Khanna
 */
public class DataSet {

	/**
	 * randomly generated test set of equal length strings
	 */
	private String[] words;
	
	/**
	 * name used for storage in Database and creating data directory
	 */
	private String name;
	
	/**
	 * Generates a set of random data.
	 * Generates a set of words to be used in BMCB testing. 
	 * Each item will be a lower case string of the given size
	 * consisting of letters a-z.
	 * @param name Unique name used for dataset storage
	 * @param size Size of dataset
	 * @param wordsize Exact size of each element in dataset
	 */
	public DataSet(String name, int size, int wordsize) 
	{
		this.name = name;
		words = new String[size];
		for(int i=0; i < size; i++) {
			words[i] = generateWord(wordsize);
		}
	}
	
	/**
	 * creates a DataSet to encapsulate an array of Strings.
	 * This is intended to be used with the DataBase class. All 
	 * strings must be the same length. No consistency checking
	 * on string length is done in this constructor. 
	 * @param name Unique name used for dataset storage
	 * @param s array of Strings to place in DataSet
	 */
	public DataSet(String name, String[] s) {
		this.name = name;
		words = s;
	}
	
	/**
	 * Prints DataSet to Standard Out.
	 */
	public void printDataSet() {
		for(int i=0; i < words.length; i++) {
			System.out.println(i+":\t"+words[i]);
		}
	}
	
	public int getSize() {
		return words.length;
	}
	
	public int getWordSize() {
		return words[0].length();
	}
	
	public String get(int i) {
		return words[i];
	}
	
	public String getName() { return name; }
	public void setName(String s) { name = s; }
	
	/**
	 * Generates a word consisting of n characters between a-z.
	 * @param size Size of string n
	 * @return random string of lower case letters of size n
	 */
	public static String generateWord(int size) {
		char[] retval = new char[size];
		Random generator = new Random();
		
		for(int i=0; i < size; i++) {
			//chose a number between 0 and 25 / 26 letters
			int r = generator.nextInt(26);
			//ASCII conversion, a is 97
			retval[i] = (char) (r + 97);
		}
		
		return new String(retval);
	}

	public String[] toArray() {
		return words;
	}
	
}
