package org.penguindreams.bmcb.filters;

import java.awt.image.BufferedImage;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;

import javax.imageio.ImageIO;

import org.penguindreams.bmcb.BMCBMain;
import org.penguindreams.bmcb.util.ApplicationLogger;
import org.penguindreams.bmcb.util.InvalidConfigException;

public class ImageMagickFilter extends AbstractFilter {

	
	private String[] args;
	
	public ImageMagickFilter(String[] args) {
		super();
		this.args = args;
	}
	
	@Override
	public void filterImage(BufferedImage img) {
		
		log.trace("Starting ImageMagick Edge Filter");
		ApplicationLogger.writeArrayToDebug(log, args);
		
		try {
			log.trace("Creating Tmp Files for ImageMagick Filter");
			File tmpOut = File.createTempFile("BMCB", ".png");
			File tmpIn = File.createTempFile("BMCB", ".png");
			
			FileOutputStream oStream = new FileOutputStream(tmpOut);
			
			ImageIO.write(img,"png",oStream);
			oStream.flush();
			oStream.close();
					
			String magickPath = BMCBMain.config.getConfigValue("cp_imagemagick");
			String magickConvert = new File(magickPath,"convert").toString();
			
			log.trace("Running ImageMagick Edge Filter");
			String[] cmd = new String[args.length + 3];
			cmd[0] = magickConvert;
			cmd[1] = tmpOut.toString();
			cmd[cmd.length-1] = tmpIn.toString();
			for(int i=0; i < args.length; i++) {
				cmd[i+2] = args[i];
			}
			
			log.debug("Running Command:");
			ApplicationLogger.writeArrayToDebug(log,cmd);
			
			Process p = Runtime.getRuntime().exec(cmd);
			
			BufferedReader stdout = new BufferedReader(new InputStreamReader(p.getInputStream()));
			BufferedReader stderr = new BufferedReader(new InputStreamReader(p.getErrorStream()));
			
			String s = null;
			while( (s = stdout.readLine()) != null) {
				log.trace("ImageMagick: " + s);
			}
			String e = null;
			while( (e = stderr.readLine()) != null) {
				log.error("ImageMagick: " + e);
			}
			
			p.waitFor();
			
			log.trace("External ImageMagick Process Complete. Loading Image");
			BufferedImage imgIn = ImageIO.read(tmpIn);
			
			//redraw it into existing BufferedImage object
			img.getGraphics().drawImage(imgIn,0,0,null);
					
		} 
		catch (IOException e) {
			log.error("Input/Ouput Problem",e);
		}
		catch (InvalidConfigException e2) {
			log.fatal("cp_imagemagick must be defined in bmcb.config");
		} 
		catch (InterruptedException e3) {
			log.fatal("Image Magick Process Interrurpted",e3);
		}


	}

}
