package org.penguindreams.bmcb.generators;

import java.io.File;

import org.apache.log4j.Logger;
import org.penguindreams.bmcb.data.DataSet;
import org.penguindreams.bmcb.util.ApplicationLogger;

public abstract class AbstractGenerator {

	
	protected DataSet data;
	
	protected File outdir;
	
	protected Logger log;
	
	public AbstractGenerator(DataSet data, File outdir) {
		this.data = data;
		this.outdir = outdir;
		log = ApplicationLogger.getLogger(this.getClass());
	}
	
	/**
	 * clears the current directory and generates output for a given dataset.
	 * <b>This will clean the current directory before producing the new set</b>
	 * Override this function to add generation functionality, but be sure to call
	 * super() to clear the output directory.
	 */
	public void generateSet() throws GenerationException {
		
		//create directory if it doesn't exist
		if(!outdir.exists() && !outdir.mkdir()) {
			 log.fatal("Could Not Create Output Directory: "+outdir);
			 System.exit(13);
		}
		
		//sanity
		if(!outdir.isDirectory()) {
			throw new GenerationException("Output directory " + outdir + 
					" either does not exist or is not a directory");
		}
		
		
		//clear directory of files
		log.debug("Clearing Output Directory " + outdir);
		File[] children = outdir.listFiles();
		for(int i=0; i< children.length; i++) {
			if(!children[i].delete()) {
				log.warn("Could not delete " + children[i]);
			}
		}
	}
	
}
