/*
 * Used to display segmented images for testing  
 */
package org.penguindreams.bmcb.gui;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import org.apache.log4j.Logger;
import org.penguindreams.bmcb.analyzers.AbstractAnalyzer;
import org.penguindreams.bmcb.analyzers.AnalysisNotSupportedException;
import org.penguindreams.bmcb.analyzers.AnalyzerNotFoundException;
import org.penguindreams.bmcb.analyzers.GuessingAnalyzer;
import org.penguindreams.bmcb.analyzers.OcradAnalyzer;
import org.penguindreams.bmcb.data.DataBase;
import org.penguindreams.bmcb.filters.AbstractFilter;
import org.penguindreams.bmcb.filters.ConvolveEdgeDetectFilter;
import org.penguindreams.bmcb.filters.ImageMagickFilter;
import org.penguindreams.bmcb.segmentators.AbstractSegmentator;
import org.penguindreams.bmcb.segmentators.EvenLengthSegmentator;
import org.penguindreams.bmcb.segmentators.LineRecSegmentator;
import org.penguindreams.bmcb.util.ApplicationLogger;
import org.penguindreams.bmcb.util.ImageLoader;

@SuppressWarnings("serial")
public class SegmentViewer extends JSplitPane implements ListSelectionListener, ActionListener {


	private ImagePane img_panel;
	
	private JList lst_set;
	
	private JComboBox cb_set_select, cb_seg_select, cb_filter_select, cb_analyzers;
	
	private DataBase db;
	
	public static final String[] SEGMENTATORS = {"LineRecSegmentator","EvenLengthSegmentator"};
	
	public static final String[] FILTERS = {"ConvolveEdgeDetectFilter", "ImageMagickEdgeFilter"}  ;
	
	public static final String[] ANALYZERS = { "GuessingAnalyzer","OcradAnalyzer" };
	
	private Logger log;
	
	private JButton b_analyze;
	
	
	public SegmentViewer() {
		super();
		
		log = ApplicationLogger.getLogger(this.getClass());
		
		db = new DataBase();
		
		img_panel = new ImagePane();
		lst_set = new JList();
		
		cb_set_select = new JComboBox();
		cb_seg_select = new JComboBox();
		cb_filter_select = new JComboBox();
		
		b_analyze = new JButton("Analyze");
		b_analyze.setEnabled(false);
		b_analyze.addActionListener((this));
		cb_analyzers = new JComboBox();
		JPanel p_bottom = new JPanel();
		p_bottom.setLayout(new FlowLayout());
		p_bottom.add(b_analyze);
		initalizeComboBox(cb_analyzers, ANALYZERS);
		p_bottom.add(cb_analyzers);
		
		setOrientation(JSplitPane.HORIZONTAL_SPLIT);
		setDividerLocation(0.3);
		setLeftComponent(createSidePane(
				createTopOptionPanel("DataSet",cb_set_select,db.getDataSets()),
				new JScrollPane(lst_set),null));
		
		//tacking on filter ComboBox
		JPanel p = createTopOptionPanel("Segmentator",cb_seg_select,SEGMENTATORS);
		p.add(new JLabel("Filter"));
		initalizeComboBox(cb_filter_select,FILTERS);
		p.add(cb_filter_select);
		
		setRightComponent(createSidePane(p,img_panel,p_bottom));
		
		lst_set.addListSelectionListener(this);
		
		
		
	}
	
	private JPanel createSidePane(JComponent top, JComponent center, JComponent bottom) {
		JPanel r = new JPanel();
		r.setLayout(new BorderLayout());
		if(top != null) {
			r.add(top,BorderLayout.NORTH);
		}
		if(center != null) {
			r.add(center,BorderLayout.CENTER);
		}
		if(bottom != null) {
			r.add(bottom,BorderLayout.SOUTH);
		}
		return r;
	}
	
	private JPanel createTopOptionPanel(String name, JComboBox cb, Object[] set) {
		JPanel r = new JPanel();
		r.setLayout(new FlowLayout());
		
		r.add(new JLabel(name));
		initalizeComboBox(cb, set);
		r.add(cb);

		return r;
	}

	private void initalizeComboBox(JComboBox cb, Object[] set) {
		//pad the set with a blank at the top
		cb.addItem("");
		
		//populate box
		for(Object o : set) {
			cb.addItem(o);
		}
		cb.addActionListener(this);		
	}

	//EVENT Listeners ------
	
	public void valueChanged(ListSelectionEvent o) {
		if(o.getSource() == lst_set && !o.getValueIsAdjusting() ) {
			validateSegmentView();
		}		
	}
	
	public void actionPerformed(ActionEvent e) {
		log.trace("Action Performed Event: " + (e));
		
		//deal with a change in datasets
		if(e.getSource() == cb_set_select) {
			log.trace("Combo Set Box Change: " + e);
			
			//make sure we didn't select the empty box
			if(!cb_set_select.getSelectedItem().equals("")) {
				lst_set.setListData(db.getDataSet(cb_set_select.getSelectedItem().toString()).toArray());
			}
		}
		else if(e.getSource() == cb_analyzers) {
			b_analyze.setEnabled(!cb_analyzers.getSelectedItem().equals(""));
		}
		else if(e.getSource() == b_analyze) {
			try {
				AbstractAnalyzer analyzer = null;
				//select our analyzer
				if(cb_analyzers.getSelectedItem().equals("OcradAnalyzer")) {
					analyzer = new OcradAnalyzer();
				}
				else if(cb_analyzers.getSelectedItem().equals("GuessingAnalyzer")) {
					analyzer = new GuessingAnalyzer();
				}
				
				//perform analyzis
				//  if we loaded a filter, it will all ready be in the img object. 
				//  we just need to worry about the segmentator here
				AbstractSegmentator segger = getCurrentSegmentator(ImageLoader.createBufferedImage(img_panel.getImage()),db.getDataSet(cb_set_select.getSelectedItem().toString()).getWordSize());
				char[] r = null;
				if(segger == null) {
					r = analyzer.analyzeImage(img_panel.getImage());
				}
				else {
					r = analyzer.analyzeSegments(segger.createSegments());						
				}

				showAnalysisBox(lst_set.getSelectedValue().toString().toCharArray(), r);
			}
			catch (AnalyzerNotFoundException anf) {
				log.error("Analyzer Not Found",anf);
				JOptionPane.showMessageDialog(new JFrame(),"Analyzer Not Found","Analyzer Not Found Error",JOptionPane.ERROR_MESSAGE);	
			}
			catch (AnalysisNotSupportedException ans) {
				log.error("Analysis Not Supported",ans);
				JOptionPane.showMessageDialog(new JFrame(),"Analysis Not Supported","Analysis Not Supported",JOptionPane.ERROR_MESSAGE);	
			}
		}
		
		//in all cases, we need to redraw the view
		if(e.getSource() == cb_set_select || e.getSource() == cb_seg_select || e.getSource() == cb_filter_select) {
			validateSegmentView();	
		}
	}
	
	//end EVENT Listeners
	
	/**
	 * updates the view for the current image and segmentator.
	 * This function should be called on event changes to update the view
	 * to reflect the current segmentator (or the image by itself if no
	 * segmentator has been selected).
	 */
	private void validateSegmentView() {
		//big IO try
		try {
			//no image selected
			if(lst_set.getSelectedValue() == null) {
				log.trace("No Image Selected. Blank the window");
				img_panel.setImage((File) null);
			}
			//image selected
			else {
				//load up that image
				File fimg = ImageLoader.constructFilePath(db.getDataSet(cb_set_select.getSelectedItem().toString()), 
						lst_set.getSelectedValue().toString());
				BufferedImage img = ImageLoader.loadImageFromFile(fimg);
				
				log.debug("Loading File " +img.toString());
				
				//check to see which Filter is selected
				if(cb_filter_select.getSelectedItem().equals("")) {
					log.trace("No Filter Selected");
				}
				else if(cb_filter_select.getSelectedItem().equals("ConvolveEdgeDetectFilter")) {
					log.trace("ConvolveEdgeDetectFilter Selected");
					AbstractFilter filter = new ConvolveEdgeDetectFilter();
					filter.filterImage(img);
				}
				else if (cb_filter_select.getSelectedItem().equals("ImageMagickEdgeFilter")) {
					log.trace("ImageMagickEdgeFilter Selected");
					String args[] = new String[] {"-edge","1"};
					AbstractFilter filter = new ImageMagickFilter(args);
					filter.filterImage(img);
				}
				
				//check to see which segmentator is selected
				AbstractSegmentator segger = getCurrentSegmentator(img,db.getDataSet(cb_set_select.getSelectedItem().toString()).getWordSize());
				if(segger == null) {
					img_panel.setImage(img);
				}
				else {
					ImageLoader.drawAxes(img,segger.getSegmentAxes());
					img_panel.setImage(img);					
				}

			}
		}
		catch(IOException e) {
			log.error("Could Not Load Image. I/O probem " + e.getMessage());
		}
	}
	
	/**
	 * Loads the appropriate segmentator base on the combo box selection. 
	 * @param image Image to be segmentated
	 * @param wordSize The size of the words in the current dataset
	 * @return A derived AbstractSegmentator or null 
	 */
	private AbstractSegmentator getCurrentSegmentator(BufferedImage image, int wordSize) {
		//check to see which segmentator is selected
		if(cb_seg_select.getSelectedItem().equals("")) {
			log.trace("An Image is selected but no Segmentator. Loading the Image by itself.");
			return null;
		}
		else if(cb_seg_select.getSelectedItem().equals("LineRecSegmentator")) {
			log.trace("Loading LinRecSegmentator to process Image");
			return new LineRecSegmentator(image);
		}
		else if(cb_seg_select.getSelectedItem().equals("EvenLengthSegmentator")) {
			log.trace("Loading EvenLengthSegmentator to process Image");
			return new EvenLengthSegmentator(image,wordSize);
		}
		
		return null;
	}
	
	
	private void showAnalysisBox(char[] expected, char[] results) {
		
		
		int correct = 0;
		for(int i=0; i < results.length; i++) {
			if(expected.length >= i+1 && 
					Character.toLowerCase(expected[i]) == Character.toLowerCase(results[i])) {
				correct++;
			}
		}
		correct -= Math.abs(expected.length - results.length);
		correct = (correct < 0) ? 0 : correct;
		float acc = (float)correct / (float)expected.length;
		
		String msg = "Expected Result: " + new String(expected) + "\n" +
		   "Analysis Result: " + new String(results) + "\n" + 
		   "Accuracy: " + acc;
		log.trace(msg);
		JOptionPane.showMessageDialog(new JFrame(), msg);
	}
}











