package org.penguindreams.bmcb.segmentators;

import java.awt.image.BufferedImage;
import org.apache.log4j.Logger;
import org.penguindreams.bmcb.util.ApplicationLogger;

public abstract class AbstractSegmentator {

	protected BufferedImage img;
	
	protected int width, height;
	
	protected Logger log;
	
	public AbstractSegmentator(BufferedImage img) {
		
		log = ApplicationLogger.getLogger(this.getClass());
		
		this.img = img; 
		//make sure the image is loaded
		if(img != null) {
			while(img.getWidth(null) == -1 || img.getHeight(null) == -1) {
				continue;
			}
			width = img.getWidth(null);
			height = img.getHeight(null);
		}
		else {
			width = height = -1;
		}
	}
	
	public BufferedImage[] createSegments() {
		return splitOnAxes(getSegmentAxes());
	}
	
	public abstract int[] getSegmentAxes();

	
	protected BufferedImage[] splitOnAxes(int[] axes) {
		
		BufferedImage[] retval = new BufferedImage[axes.length + 1];
		
		//we'll do the first one here / 0 - first axis
		retval[0] = img.getSubimage(0, 0, axes[0], height);
		
		//now we'll do the rest
		for(int i=0; i < axes.length; i++) {
			
			//get the endpoint, then subtract off our location to get the width
			int endWidth = (i != axes.length - 1) ? axes[i+1] : width;
			endWidth = endWidth - axes[i];
			
			retval[i+1] = img.getSubimage(axes[i], 0, endWidth, height);
		}
		
		return retval;
	}
}
