package org.penguindreams.bmcb.segmentators;

import java.awt.image.BufferedImage;

public class LineRecSegmentator extends AbstractSegmentator {

	public LineRecSegmentator(BufferedImage img) {
		super(img);
	}

	@Override
	public int[] getSegmentAxes() {
	
		double[] devis = new double[width];
		for(int i=0; i < width; i++) {
			int[] px = new int[height]; //store each pixel in column
			for(int j=0; j < height; j++) {
				px[j] = img.getRGB(i, j);
			}
			devis[i] = calculateStandardDeviation(px);
			log.trace("Standard Deviation for column " + i + " is " + devis[i]);
		}
		
		return new int[] {10,20,30}; //for testing
	}
	

	
	/**
	 * calculates the standard deviation from a set of integers.
	 * Finds the average of a set of integers, then finds the 
	 * sum of the integers multiplied by the average, then divides
	 * the sum by the number of elements minus one and finally, takes
	 * the result of that division and finds the square root of it.  
	 * @param i set of integers to find standard deviation of
	 * @return the standard deviation of the set of input integers
	 */
	private double calculateStandardDeviation(int[] i) {
		
		//you need at least three elements
		if(i.length < 2) { return Double.NaN; }
		
		//calculate average
		double avg = 0;
		for(int x=0; x < i.length; x++) {
			avg += i[x];
		}
		avg = avg / i.length;
		
		//sum of squares
		double sum = 0;
		for(int x=0; x < i.length; x++) {
			final double d = i[x] * avg;
			sum += d * d;
		}
		
		//return the standard deviation
		return Math.sqrt(sum / i.length - 1 );
	}

}
