package org.penguindreams.bmcb.util;

import java.io.File;
import java.util.ArrayList;

import org.apache.log4j.Logger;
import org.penguindreams.bmcb.BMCBMain;
import org.penguindreams.bmcb.data.ExperimentResult;

public abstract class AbstractResultHandler {
	
	protected String title;
	
	protected ArrayList<ExperimentResult> data;
	
	protected Logger log;
	
	public String getTitle() { return title; }
	public void setTitle(String s) { title = s; }
	
	protected File outDir;
	
	public void addResult(ExperimentResult f) { data.add(f); }
	public ExperimentResult[] getResults(){ return data.toArray(new ExperimentResult[0]); }
	
	public AbstractResultHandler(String title) {
		this.title = title;
		data = new ArrayList<ExperimentResult>();
		log = ApplicationLogger.getLogger(this.getClass());
		
		try {
			outDir = new File(BMCBMain.config.getConfigValue("cp_results"));
		} catch (InvalidConfigException e) {
			// TODO kill app
			log.fatal("Could not find config attribute for cp_results");
		}
		if(!outDir.isDirectory()) {
			log.fatal("Results Directory " + outDir.getAbsolutePath() + " is not a directory");
			//TODO: kill app
		}
	}
	
	public abstract void generateGraph();
	
	/**
	 * Converts a floating point number to a percentage with a one decimal degree of accuracy.
	 * For example, 0.457643 would be converted to 45.7
	 * @param f Percentage to convert
	 * @return floating point number between 0.0 and 100.0
	 */
	protected float convertFloatToPercentage(float f) {
		//turn this into an interger percentage betwen 0 and 1000
		int a = (int) (f * 1000.0);
		//then turn it back into %##.#
		return ((float)a) / 10;
	}
}
