/*
 * Class to handle config files
 * 
 * Created on Aug 6, 2007
 * Sumit Khanna
 */
package org.penguindreams.bmcb.util;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.util.HashMap;

import org.apache.log4j.Logger;

import org.penguindreams.bmcb.util.InvalidConfigException;

/**
 * @author s21574k
 */
public class ConfigMap {

	private String configfile;
	
	private HashMap<String,String> configmap;
	
	/**
	 * Creates a configuration object from a colon (:) delimited file.
	 * File should have key:value, one per line. # indicates comments
	 * @param file configuration file
	 * @throws IOException thrown if file can not be found or of there is an error reading it.
	 */
	public ConfigMap(String file) throws IOException {
		
		configmap = new HashMap<String,String>();
		configfile = file;		
		Logger log = ApplicationLogger.getLogger(this.getClass());
		
		log.info("Loading Configuration From " + file);
		BufferedReader fd = new BufferedReader(new FileReader(configfile));
		String line = null;
		while( (line = fd.readLine()) != null ) {
			String[] parts = line.split(":");
			//ignore # / comments
			if(parts.length == 2 && parts[0].trim().charAt(0) != '#') {
				//warn upon duplicate elements
				if(configmap.get(parts[0].trim()) != null) {
					log.warn("Redeclaring Configuration Element \"" + parts[0].trim() + "\" in the file " +
							file + ". The old value will be overwritten.");
				}
				configmap.put(parts[0].trim(),parts[1].trim());
				log.debug("Added Key:" + parts[0].trim() + " Value:" +parts[1].trim());
			}
		}
		log.info("Configuration Loaded");
	}
	
	/**
	 * determines the value for a given key
	 * @param key
	 * @return either a value or null if there is not mapping.
	 */
	public String getConfigValue(String key) throws InvalidConfigException {
		String retval =  configmap.get(key);
		if(retval == null) {
			throw new InvalidConfigException("Could not find config item: " + key);
		}
		return retval;
	}
	
	/**
	 * finds all configuration keys
	 * @return String[] of all configuration keys
	 */
	public String[] getKeys() {
		Object[] okeys = configmap.keySet().toArray();
		String[] skeys = new String[okeys.length];
		for(int i=0; i<okeys.length; i++) {
			skeys[i] = okeys[i].toString();
		}
		return skeys;
	}

	/**
	 * generates an array of all config values.
	 * @return String[] of all configuration values.
	 */
	public String[] getAllValues() {
		Object[] c = configmap.values().toArray();
		String[] retval = new String[c.length];
		for(int i=0; i<retval.length; i++) {
			retval[i] = c[i].toString();
		}
		return retval;
	}
}








