package org.penguindreams.bmcb.util;

import java.awt.Image;
import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;

import javax.imageio.ImageIO;

import org.apache.log4j.Logger;
import org.penguindreams.bmcb.BMCBMain;
import org.penguindreams.bmcb.data.ExperimentResult;


public class GoogleGraphCreator extends AbstractResultHandler {


	
	public static final String GOOGLE_URL = "http://chart.apis.google.com/chart";
	
	
	public GoogleGraphCreator(String title) {
		super(title);
	} 
	

	
	
	public void generateGraph() {
		Image graph = null ;
		
		try {
			graph = ImageIO.read(new URL( (GOOGLE_URL + formatDataURL()).replace(" ", "%20")  ));
			
			ImageIO.write(ImageLoader.createBufferedImage(graph), "PNG", new File(outDir,title + ".png"));
		} catch (MalformedURLException e) {
			// TODO Auto-generated catch block
			log.fatal("Bad URL",e);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			log.fatal("Input/Output Error When Rendering Google Graph",e);
		}
		
		
		
	}
	
	
	private String formatDataURL() {
		String retval = "?";
		
		retval += "chtt=" + title.replace(" ","%20") + "&"; //title

		retval += "chdl=Percentage of Words Correct|Percentage of Letters Correct&"; //legend
		
		int barSize = 20, barSpacing = 20, groupSpacing = 30; //defaults
		
		try {
			retval += "cht=" + BMCBMain.config.getConfigValue("gchart_type") + "&"; //chart type
			
			retval += "chco=" +BMCBMain.config.getConfigValue("gchart_colors") + "&"; //colors
			
			retval += "chs=" + BMCBMain.config.getConfigValue("gchart_size") + "&"; //size
			
			barSize = Integer.parseInt( BMCBMain.config.getConfigValue("gchart_barSize") );
			
			barSpacing = Integer.parseInt( BMCBMain.config.getConfigValue("gchart_barSpacing") );
			
			groupSpacing = Integer.parseInt( BMCBMain.config.getConfigValue("gchart_groupSpacing") );

		} catch (InvalidConfigException ce) {
			log.fatal("Configuration Error in Google Chart API " + ce.getMessage());
			System.exit(9);
		}
		catch (NumberFormatException e) {
			log.fatal("gchart setting expected interger",e);
			System.exit(9);
		}
				
		retval += "chbh=" + barSize + "," + barSpacing + "," + groupSpacing + "&";
		
				String fDataChars= ""; //formatted data
		String fDataWords= "";
		String fDataAxis="";
		for(ExperimentResult r : data) {
			fDataChars += convertFloatToPercentage(r.getCharacterAccuracy()) + ",";
			fDataWords += convertFloatToPercentage(r.getWordAccuracy()) + ",";
			fDataAxis += r.getAnalyzerName()+"|";
		}
		//trim off those trailing commas
		fDataChars = fDataChars.substring(0, fDataChars.length()-1);
		fDataWords = fDataWords.substring(0, fDataWords.length()-1);
		//fDataAxis = fDataWords.substring(0, fDataAxis.length()-2);
		
		retval += "chd=t:" + fDataWords + "|" + fDataChars + "&";
		retval += "chxt=x,y&chxl=0:|"+ fDataAxis  +"&";
		
		log.info("Google Graph URL " + GOOGLE_URL + retval);
		
		return retval;
	}
	

	
}
