package org.penguindreams.bmcb.workflows;

import java.awt.image.BufferedImage;
import java.io.IOException;

import org.apache.log4j.Logger;
import org.penguindreams.bmcb.BMCBMain;
import org.penguindreams.bmcb.analyzers.AbstractAnalyzer;
import org.penguindreams.bmcb.analyzers.AnalysisNotSupportedException;
import org.penguindreams.bmcb.data.DataBase;
import org.penguindreams.bmcb.data.DataSet;
import org.penguindreams.bmcb.data.ExperimentResult;
import org.penguindreams.bmcb.filters.AbstractFilter;
import org.penguindreams.bmcb.segmentators.AbstractSegmentator;
import org.penguindreams.bmcb.segmentators.EvenLengthSegmentator;
import org.penguindreams.bmcb.util.ApplicationLogger;
import org.penguindreams.bmcb.util.GoogleGraphCreator;
import org.penguindreams.bmcb.util.ImageLoader;
import org.penguindreams.bmcb.util.InvalidConfigException;

public abstract class AbstractWorkflow {
	
	protected Logger log;
	protected DataBase db;
	
	protected String genpath = null,setdirs = null,tmpdirs = null; 
	
	protected AbstractWorkflow() {
		log = ApplicationLogger.getLogger(this.getClass());
		db = new DataBase();
		
		//load config
		log.info("Loading Workflow Configuration");
		try {
			genpath = BMCBMain.config.getConfigValue("cp_generators");
			setdirs = BMCBMain.config.getConfigValue("cp_setdirs");
			tmpdirs = BMCBMain.config.getConfigValue("cp_tmpdirs");
		} catch (InvalidConfigException e) {
			log.fatal("Could Not Initalize Workflow Configuration");
			log.fatal(e.getMessage());
			System.exit(10);
		}
		log.info("Config Loading Complete");
	}
	
	public abstract void execute();
	
	protected ExperimentResult[] runSpecific(String desc,  AbstractSegmentator segger, AbstractFilter filter, AbstractAnalyzer an) throws IOException, AnalysisNotSupportedException {

		String[] sets = db.getDataSets();
		
		ExperimentResult[] retval = new ExperimentResult[sets.length];
		int retIdx = 0;
		
		for(String setName : sets) {

			DataSet s = db.getDataSet(setName);
			
			ExperimentResult gsr = new ExperimentResult(setName,setName,desc);
			for(int x=0; x<s.getSize(); x++) {
				String element = s.get(x);
				BufferedImage testImage = ImageLoader.createBufferedImage(ImageLoader.loadImageFromFile(ImageLoader.constructFilePath(s, element)));
				
				if(filter != null) {
					filter.filterImage(testImage);
				}
				
				//This is kinda messy since I tried to move it into its own function due to segmentation
				if(segger == null) {
					log.trace("No Segmentator Given");	
				}
				else if(segger instanceof EvenLengthSegmentator) {
					log.trace("Even Length Segmentator Given");
					segger = new EvenLengthSegmentator(testImage,s.getWordSize());
				}
				else {
					log.warn("Unknown Segmentator. I have no idea what to do. Bypassing Segmentation");
					segger = null;
				}
				
				//we check for segmentation here too
				char r[] = null;
				if(segger != null) {
					r = an.analyzeSegments(segger.createSegments());
				}else {
					r = an.analyzeImage(testImage);
				}
				
				gsr.addResult(element.toCharArray(),r);
			}
			log.info(gsr.toString());
			retval[retIdx++] = gsr;
	
		}
		return retval;
	}
	
}
