/**
 * This workflow is used to generate all the Captcha datasets.
 * It both randomly generates 
 */
package org.penguindreams.bmcb.workflows;

import java.io.File;

import org.penguindreams.bmcb.data.DataSet;
import org.penguindreams.bmcb.generators.CommandLineGenerator;

public class DatasetGeneratorWorkflow extends AbstractWorkflow {

	
	public DatasetGeneratorWorkflow() {
		super();		
	}
	
	/**
	 * Array of captcha generator names and their program locations.
	 */
	private final String[][] ENGINES = {
			{"CaptchaPHP","/captchaphp/captcha.php"},
			{"gotcha","/gotcha/captcha_image.php"},
			{"freecap","/freecap/freecap.php" },
			{"trivial","/trivial/trivial.php"}
	};
	
	/**
	 * Types of sets to generate.
	 * The first element indicates the word size.
	 * The second element indicates the set size.
	 */
	/*private final int[][] SETS = {
			{5,100},
			{10,100}
	};*/
	
	/**
	 * size of individual captcha words.
	 */
	public final int WORD_SIZE = 5;
	
	/**
	 * number of captchas in each set
	 */
	public final int SET_SIZE = 1000;
	
	
	
	@Override
	public void execute() {	
		for(String s[] : ENGINES) {
			log.info("Generating Sets for " + s[0]);

			log.trace("Beginning to Process Set: " + s[0]);
			DataSet ds = new DataSet(s[0],SET_SIZE,WORD_SIZE);
			if(!db.writeDataSet(ds)) {
				log.warn("Dataset "+s[0]+" exists. Loading existing from Database");
				ds = db.getDataSet(s[0]);
			}
			CommandLineGenerator clg = new CommandLineGenerator(ds,genpath + s[1],
					new File(setdirs,ds.getName()));
			clg.generateSet();
			log.info(s[0] + " Set Generation Complete");
		}
	}
}
