package org.penguindreams.bmcb.workflows;

import java.io.IOException;

import org.penguindreams.bmcb.analyzers.AbstractCommandLineAnalyzer;
import org.penguindreams.bmcb.analyzers.AnalysisNotSupportedException;
import org.penguindreams.bmcb.analyzers.AnalyzerNotFoundException;
import org.penguindreams.bmcb.analyzers.GocrAnalyzer;
import org.penguindreams.bmcb.analyzers.OcradAnalyzer;
import org.penguindreams.bmcb.data.ExperimentResult;
import org.penguindreams.bmcb.filters.ImageMagickFilter;
import org.penguindreams.bmcb.util.GoogleGraphCreator;

public class ImageMagickFilterTestWorkflow extends AbstractWorkflow {

	public ImageMagickFilterTestWorkflow() {
		super();
	}
	
	public static final String[][] magicArgs = {
		{"-modulate","180,100"},
		{"-modulate","110,100"},
		{"-modulate","140,5"},
		{"-noise","1" },
		{"-modulate","160,100"}
	};
	
	public static final String[] descriptions = {
		"Modulate 180,100",
		"Modulate 110,100",
		"Modulate 140,5",
		"Reduce Noise 1",
		"Modulate 160,100"
	};
	
	@Override
	public void execute() {
		
		try{
			AbstractCommandLineAnalyzer[] ans = { new GocrAnalyzer(), new OcradAnalyzer() };
			
			for(AbstractCommandLineAnalyzer a : ans) {
				for(int x=0; x<magicArgs.length; x++) {				
					
					ExperimentResult[] r = runSpecific("Imagemagick Filter Test",
							null,new ImageMagickFilter(magicArgs[x]),a);
					
					String graphTitle = a.getName() + " Analysis with Image Magick " + descriptions[x];
					GoogleGraphCreator graph = new GoogleGraphCreator(graphTitle);
					
					for(ExperimentResult ri : r) {
						graph.addResult(ri);
					}
					graph.generateGraph();
				}		
			}

		}
		catch(Exception e) {
			String msg = "";
			if(e instanceof AnalyzerNotFoundException) {
				msg = "Analyzer Not Found";
			}
			else if(e instanceof IOException) {
				msg = "I/O Problem";
			}
			else if(e instanceof AnalysisNotSupportedException) {
				msg = "Analysis Not Supported";
			}
			else {
				msg = "Unexpected Error";
			}
			log.fatal(msg,e);
		}

	}

}
