package org.penguindreams.bmcb.workflows;

import java.io.IOException;

import org.penguindreams.bmcb.analyzers.AbstractCommandLineAnalyzer;
import org.penguindreams.bmcb.analyzers.AnalysisNotSupportedException;
import org.penguindreams.bmcb.analyzers.AnalyzerNotFoundException;
import org.penguindreams.bmcb.analyzers.GocrAnalyzer;
import org.penguindreams.bmcb.analyzers.GuessingAnalyzer;
import org.penguindreams.bmcb.analyzers.OcradAnalyzer;
import org.penguindreams.bmcb.data.ExperimentResult;
import org.penguindreams.bmcb.segmentators.EvenLengthSegmentator;
import org.penguindreams.bmcb.util.GoogleGraphCreator;

public class TrivialTestWorkflow extends AbstractWorkflow {

	
	
	public TrivialTestWorkflow() {
		super();
	}
	
	
	@Override
	public void execute()  {
		
		try{
			 
			
			ExperimentResult[] results;
			//Trivial Guessing Analyzer 
			results = runSpecific("In this experiment, the analyzer randomly guesses at the correct word",
					new EvenLengthSegmentator(null),null,new GuessingAnalyzer());
			GoogleGraphCreator  graph = new GoogleGraphCreator("Trivial Guessing");
			for(ExperimentResult ri : results) {
				graph.addResult(ri);
			}
			graph.generateGraph();
			
			AbstractCommandLineAnalyzer[] ans = { new GocrAnalyzer(), new OcradAnalyzer() };
			
			for(AbstractCommandLineAnalyzer a : ans) {
				
				//OCR Analyzers
				results = runSpecific("In this experiment "+a.getName()+", an open source OCR program, is used to analyze the raw unfiltered image",
						null,null,a);
				graph = new GoogleGraphCreator(a.getName() + " OCR Test");
				for(ExperimentResult ri : results) {
					graph.addResult(ri);
				}				
				graph.generateGraph();
				
			}

		}
		catch(Exception e) {
			String msg = "";
			if(e instanceof AnalyzerNotFoundException) {
				msg = "Analyzer Not Found";
			}
			else if(e instanceof IOException) {
				msg = "I/O Problem";
			}
			else if(e instanceof AnalysisNotSupportedException) {
				msg = "Analysis Not Supported";
			}
			else {
				msg = "Unexpected Error";
			}
			log.fatal(msg,e);
		}
	}


}
